
export default class SettingUtils {
  /**
   * @description 生成页面查询语句
   * @return {String}  scripts       自定义脚本
   * @return {String}  searches      搜索条件
   * @return {Object}  setting       页面设置
   * @return {Array}   columns       显示字段
   */
  static getDebugSql (setting, scripts, columns, searches = []) {
    let sql = ''
    let error = ''
    let _dataresource = ''
    let _customScript = ''
    let arr_field = columns.map(item => item.field).join(',')

    if (scripts.length > 0) {
      scripts.forEach(item => {
        _customScript += `
          ${item.sql}
        `
      })
    }

    if (setting.interType === 'system' && setting.execute !== 'false') {
      _dataresource = setting.dataresource || ''
    }
    
    _dataresource = _dataresource.replace(/@\$|\$@/ig, '')
    _customScript = _customScript.replace(/@\$|\$@/ig, '')

    if (_customScript) {
      _customScript = `declare @ErrorCode nvarchar(50),@retmsg nvarchar(4000),@UserName nvarchar(50),@FullName nvarchar(50),@login_city nvarchar(50) select @ErrorCode='',@retmsg =''
        ${_customScript}
      `
    }

    // 外联数据库替换
    if (window.GLOB.externalDatabase !== null) {
      _dataresource = _dataresource.replace(/@db@/ig, window.GLOB.externalDatabase)
      _customScript = _customScript.replace(/@db@/ig, window.GLOB.externalDatabase)
    }

    if (window.GLOB.urlFields) {
      window.GLOB.urlFields.forEach(field => {
        let reg = new RegExp('@' + field + '@', 'ig')
        _dataresource = _dataresource.replace(reg, '0')
        _customScript = _customScript.replace(reg, '0')
      })
    }
    
    
    // 正则替换
    let _regoptions = []
    let _fields = []

    searches.forEach(item => {
      if (!item.field) return
      if (item.datefield) {
        _regoptions.push({
          var: new RegExp('@' + item.datefield, 'ig'),
          reg: new RegExp('@' + item.datefield + '@', 'ig')
        })
        _regoptions.push({
          var: new RegExp('@' + item.datefield + '1', 'ig'),
          reg: new RegExp('@' + item.datefield + '1@', 'ig')
        })
      }
      if (['dateweek', 'datemonth', 'daterange'].includes(item.type)) {
        _regoptions.push({
          var: new RegExp('@' + item.field, 'ig'),
          reg: new RegExp('@' + item.field + '@', 'ig')
        })
        _regoptions.push({
          var: new RegExp('@' + item.field + '1', 'ig'),
          reg: new RegExp('@' + item.field + '1@', 'ig')
        })
      } else if (item.type === 'date') {
        if (_fields.includes(item.field)) {
          _regoptions.push({
            var: new RegExp('@' + item.field + '1', 'ig'),
            reg: new RegExp('@' + item.field + '1@', 'ig')
          })
        } else {
          _fields.push(item.field)
          _regoptions.push({
            var: new RegExp('@' + item.field, 'ig'),
            reg: new RegExp('@' + item.field + '@', 'ig')
          })
        }
      } else if (item.type === 'text' || item.type === 'select') {
        item.field.split(',').forEach(field => {
          _regoptions.push({
            var: new RegExp('@' + field, 'ig'),
            reg: new RegExp('@' + field + '@', 'ig')
          })
        })
      } else {
        _regoptions.push({
          var: new RegExp('@' + item.field, 'ig'),
          reg: new RegExp('@' + item.field + '@', 'ig')
        })
      }
    })

    let _search = ''

    if (setting.queryType === 'statistics' && _dataresource) {
      _regoptions.forEach(item => {
        _dataresource = _dataresource.replace(item.reg, '0')
      })

      _search = ''
    }

    let originscript = _customScript

    if (_customScript) {
      _regoptions.push({
        reg: new RegExp('@login_city@', 'ig'),
      }, {
        reg: new RegExp('@orderBy@', 'ig'),
      }, {
        reg: new RegExp('@UserName@', 'ig'),
      }, {
        reg: new RegExp('@FullName@', 'ig'),
      })
      if (setting.laypage !== 'false') {
        _regoptions.push({
          reg: new RegExp('@pageSize@', 'ig'),
        }, {
          reg: new RegExp('@pageIndex@', 'ig'),
        })
      }
      _regoptions.forEach(item => {
        _customScript = _customScript.replace(item.reg, '0')
        originscript = originscript.replace(item.reg, '0')
      })

      if (setting.varMark) {
        originscript = originscript.replace(/@ErrorCode/ig, '')
        originscript = originscript.replace(/@retmsg/ig, '')
        originscript = originscript.replace(/@UserName@/ig, '').replace(/@UserName/ig, '')
        originscript = originscript.replace(/@FullName@/ig, '').replace(/@FullName/ig, '')
        originscript = originscript.replace(/@login_city@/ig, '').replace(/@login_city/ig, '')
        originscript = originscript.replace(/@id@/ig, '').replace(/@id/ig, '')
        originscript = originscript.replace(/@bid@/ig, '').replace(/@bid/ig, '')
        originscript = originscript.replace(/@loginuid@/ig, '').replace(/@loginuid/ig, '')
        originscript = originscript.replace(/@sessionuid@/ig, '').replace(/@sessionuid/ig, '')
        originscript = originscript.replace(/@userid@/ig, '').replace(/@userid/ig, '')
        originscript = originscript.replace(/@appkey@/ig, '').replace(/@appkey/ig, '')
        originscript = originscript.replace(/@time_id@/ig, '').replace(/@time_id/ig, '')
  
        originscript = originscript.replace(new RegExp('@' + setting.varMark, 'ig'), '')

        if (/@/ig.test(originscript)) {
          error = '使用了变量标识外的字段！'
        }
      }
    }

    // 数据源处理, 存在显示列时 
    if (arr_field && _dataresource) {
      if (/\s/.test(_dataresource)) {
        _dataresource = '(' + _dataresource + ') tb'
      }

      _dataresource = `select${setting.laypage !== 'false' ?  ' top 10' : ''} ${arr_field} from (select ${arr_field} ,ROW_NUMBER() over(order by ${setting.order}) as rows from ${_dataresource} ${_search}) tmptable ${setting.laypage !== 'false' ?  'where rows > 0' : ''} order by tmptable.rows`
    }

    if (_customScript) {
      sql = `/* sql 验证 */
        ${_customScript}
        ${_dataresource}
        aaa:
        if @ErrorCode!=''
          insert into tmp_err_retmsg (ID, ErrorCode, retmsg, CreateUserID) select @time_id@,@ErrorCode, @retmsg,@UserID@
      `
    } else {
      sql = `/* sql 验证 */
        ${_dataresource}`
    }
    sql = sql.replace(/\n\s{8}/ig, '\n')

    console.info(sql)
    
    return { error, sql }
  }
}